@@ -1,20 +1,30 @@
 import re
 from copy import copy
+from typing import TypedDict
+
 from algosdk.source_map import SourceMap
 
-LOGIC_ERROR = "TransactionPool.Remember: transaction ([A-Z0-9]+): logic eval error: (.*). Details: pc=([0-9]+), opcodes=.*"
+LOGIC_ERROR = "TransactionPool.Remember: transaction (?P<txid>[A-Z0-9]+): logic eval error: (?P<msg>.*). Details: pc=(?P<pc>[0-9]+), opcodes=.*"
 
 
-def parse_logic_error(error_str: str) -> tuple[str, str, int]:
-    matches = re.match(LOGIC_ERROR, error_str)
-    if matches is None:
-        return "", "", 0
+class LogicErrorData(TypedDict):
+    txid: str
+    msg: str
+    pc: int
 
-    txid = matches.group(1)
-    msg = matches.group(2)
-    pc = int(matches.group(3))
 
-    return txid, msg, pc
+def parse_logic_error(
+    error_str: str,
+) -> LogicErrorData | None:
+    match = re.match(LOGIC_ERROR, error_str)
+    if match is None:
+        return None
+
+    return {
+        "txid": match.group("txid"),
+        "msg": match.group("msg"),
+        "pc": int(match.group("pc")),
+    }
 
 
 class LogicException(Exception):
@@ -23,6 +33,9 @@ class LogicException(Exception):
         logic_error: Exception,
         program: str,
         map: SourceMap,
+        txid: str,
+        msg: str,
+        pc: int,
     ):
         self.logic_error = logic_error
         self.logic_error_str = str(logic_error)
@@ -31,7 +44,7 @@ class LogicException(Exception):
         self.map = map
         self.lines = program.split("\n")
 
-        self.txid, self.msg, self.pc = parse_logic_error(self.logic_error_str)
+        self.txid, self.msg, self.pc = txid, msg, pc
         self.line_no = self.map.get_line_for_pc(self.pc)
 
     def __str__(self) -> str:
